@@ -1,28 +1,55 @@
 #!/usr/bin/env python
 import itertools
-from collections import namedtuple
+from typing import Callable, Dict, Iterable, List, NamedTuple, Optional, Tuple, Union
+
+from django.utils.encoding import force_str
+from django.utils.translation import get_language, override
 
 from django_countries.conf import settings
-from django.utils.encoding import force_str
-from django.utils.translation import override
 
 from .base import CountriesBase
 
 try:
-    import pyuca  # noqa
+    import pyuca  # type: ignore
+
+    collator = pyuca.Collator()
+
+    # Use UCA sorting if it's available.
+    def sort_key(item):
+        return collator.sort_key(item[1])
+
+
 except ImportError:
-    pyuca = None
     # Fallback if the UCA sorting is not available.
     import unicodedata
 
+    # Cheap and dirty method to sort against ASCII characters only.
+    def sort_key(item):
+        return (
+            unicodedata.normalize("NFKD", item[1])
+            .encode("ascii", "ignore")
+            .decode("ascii")
+        )
+
+
+CountryCode = Union[str, int, None]
+
+
+class AltCodes(NamedTuple):
+    alpha3: str
+    numeric: Optional[int]
+
+
+class CountryTuple(NamedTuple):
+    code: str
+    name: str
 
-class CountryTuple(namedtuple("CountryTupleBase", ["code", "name"])):
     def __repr__(self):
         """
         Display the repr as a standard tuple for better backwards
         compatibility with outputting this in a template.
         """
-        return "({this.code!r}, {this.name!r})".format(this=self)
+        return f"({self.code!r}, {self.name!r})"
 
 
 class Countries(CountriesBase):
@@ -33,7 +60,10 @@ class Countries(CountriesBase):
     the country ``code`` and ``name``), sorted by name.
     """
 
-    def get_option(self, option):
+    _countries: Dict[str, Union[str, Dict]]
+    _alt_codes: Dict[str, AltCodes]
+
+    def get_option(self, option: str):
         """
         Get a configuration option, trying the options attribute first and
         falling back to a Django project setting.
@@ -41,10 +71,10 @@ class Countries(CountriesBase):
         value = getattr(self, option, None)
         if value is not None:
             return value
-        return getattr(settings, "COUNTRIES_{0}".format(option.upper()))
+        return getattr(settings, f"COUNTRIES_{option.upper()}")
 
     @property
-    def countries(self):
+    def countries(self) -> Dict[str, Union[str, dict]]:
         """
         Return the a dictionary of countries, modified by any overriding
         options.
@@ -52,25 +82,39 @@ class Countries(CountriesBase):
         The result is cached so future lookups are less work intensive.
         """
         if not hasattr(self, "_countries"):
-            only = self.get_option("only")
+            only: Iterable[Union[str, Tuple[str, str]]] = self.get_option("only")
+            only_choices = True
             if only:
-                only_choices = True
+                # Originally used ``only`` as a dict, still supported.
                 if not isinstance(only, dict):
                     for item in only:
                         if isinstance(item, str):
                             only_choices = False
                             break
+            self._shadowed_names: Dict[str, List[str]] = {}
             if only and only_choices:
-                self._countries = dict(only)
+                self._countries = dict(only)  # type: ignore
             else:
                 # Local import so that countries aren't loaded into memory
                 # until first used.
                 from django_countries.data import COUNTRIES
 
-                self._countries = dict(COUNTRIES)
+                countries_dict = dict(COUNTRIES)
+                if only:
+                    self._countries = {}
+                    for item in only:
+                        if isinstance(item, str):
+                            self._countries[item] = countries_dict[item]
+                        else:
+                            key, value = item
+                            self._countries[key] = value
+                else:
+                    self._countries = countries_dict.copy()  # type: ignore
                 if self.get_option("common_names"):
-                    self._countries.update(self.COMMON_NAMES)
-                override = self.get_option("override")
+                    for code, name in self.COMMON_NAMES.items():
+                        if code in self._countries:
+                            self._countries[code] = name
+                override: Dict[str, Union[str, dict]] = self.get_option("override")
                 if override:
                     self._countries.update(override)
                     self._countries = dict(
@@ -78,46 +122,24 @@ class Countries(CountriesBase):
                         for code, name in self._countries.items()
                         if name is not None
                     )
-            if only and not only_choices:
-                countries = {}
-                for item in only:
-                    if isinstance(item, str):
-                        countries[item] = self._countries[item]
-                    else:
-                        key, value = item
-                        countries[key] = value
-                self._countries = countries
+
+                if self.get_option("common_names"):
+                    for code in self.COMMON_NAMES:
+                        if code in self._countries and code not in override:
+                            self._shadowed_names[code] = [countries_dict[code]]
+                for code, names in self.OLD_NAMES.items():
+                    if code in self._countries and code not in override:
+                        country_shadowed = self._shadowed_names.setdefault(code, [])
+                        country_shadowed.extend(names)
+
             self.countries_first = []
-            first = self.get_option("first") or []
+            first: List[str] = self.get_option("first") or []
             for code in first:
                 code = self.alpha2(code)
                 if code in self._countries:
                     self.countries_first.append(code)
         return self._countries
 
-    @property
-    def alt_codes(self):
-        if not hasattr(self, "_alt_codes"):
-            # Again, local import so data is not loaded unless it's needed.
-            from django_countries.data import ALT_CODES
-
-            self._alt_codes = ALT_CODES
-            altered = False
-            for code, country in self.countries.items():
-                if isinstance(country, dict) and (
-                    "alpha3" in country or "numeric" in country
-                ):
-                    if not altered:
-                        self._alt_codes = self._alt_codes.copy()
-                        altered = True
-                    alt_codes = list(self._alt_codes.get(code, (None, None)))
-                    if "alpha3" in country:
-                        alt_codes[0] = country["alpha3"]
-                    if "numeric" in country:
-                        alt_codes[1] = country["numeric"]
-                    self._alt_codes[code] = tuple(alt_codes)
-        return self._alt_codes
-
     @countries.deleter
     def countries(self):
         """
@@ -129,8 +151,57 @@ class Countries(CountriesBase):
             del self._countries
         if hasattr(self, "_alt_codes"):
             del self._alt_codes
+        if hasattr(self, "_ioc_codes"):
+            del self._ioc_codes
+        if hasattr(self, "_shadowed_names"):
+            del self._shadowed_names
 
-    def translate_code(self, code, ignore_first=None):
+    @property
+    def alt_codes(self) -> Dict[str, AltCodes]:
+        if not hasattr(self, "_alt_codes"):
+            # Again, local import so data is not loaded unless it's needed.
+            from django_countries.data import ALT_CODES
+
+            self._alt_codes = ALT_CODES  # type: ignore
+            altered = False
+            for code, country in self.countries.items():
+                if isinstance(country, dict) and (
+                    "alpha3" in country or "numeric" in country
+                ):
+                    if not altered:
+                        self._alt_codes = self._alt_codes.copy()
+                        altered = True
+                    alpha3, numeric = self._alt_codes.get(code, ("", None))
+                    if "alpha3" in country:
+                        alpha3 = country["alpha3"]
+                    if "numeric" in country:
+                        numeric = country["numeric"]
+                    self._alt_codes[code] = AltCodes(alpha3, numeric)
+        return self._alt_codes
+
+    @property
+    def ioc_codes(self) -> Dict[str, str]:
+        if not hasattr(self, "_ioc_codes"):
+            from django_countries.ioc_data import ISO_TO_IOC
+
+            self._ioc_codes = ISO_TO_IOC
+            altered = False
+            for code, country in self.countries.items():
+                if isinstance(country, dict) and "ioc_code" in country:
+                    if not altered:
+                        self._ioc_codes = self._ioc_codes.copy()
+                        altered = True
+                    self._ioc_codes[code] = country["ioc_code"]
+        return self._ioc_codes
+
+    @property
+    def shadowed_names(self):
+        if not getattr(self, "_shadowed_names", False):
+            # Getting countries populates shadowed names.
+            self.countries
+        return self._shadowed_names
+
+    def translate_code(self, code: str, ignore_first: List[str] = None):
         """
         Return translated countries for a country code.
         """
@@ -147,7 +218,7 @@ class Countries(CountriesBase):
         for name in names:
             yield self.translate_pair(code, name)
 
-    def translate_pair(self, code, name=None):
+    def translate_pair(self, code: str, name: Union[str, Dict] = None):
         """
         Force a country to the current activated translation.
 
@@ -155,28 +226,34 @@ class Countries(CountriesBase):
         """
         if name is None:
             name = self.countries[code]
-            if isinstance(name, dict):
-                if "names" in name:
-                    name = name["names"][0]
-                else:
-                    name = name["name"]
-        if code in self.OLD_NAMES:
+        if isinstance(name, dict):
+            if "names" in name:
+                country_name: str = name["names"][0]
+                fallback_names: List[str] = name["names"][1:]
+            else:
+                country_name = name["name"]
+                fallback_names = []
+        else:
+            country_name = name
+            fallback_names = self.shadowed_names.get(code, [])
+        language = get_language()
+        if language and language.split("-")[0] != "en" and fallback_names:
             # Check if there's an older translation available if there's no
             # translation for the newest name.
-            with override(None):
-                source_name = force_str(name)
-            name = force_str(name)
-            if name == source_name:
-                for old_name in self.OLD_NAMES[code]:
-                    with override(None):
-                        source_old_name = force_str(old_name)
-                    old_name = force_str(old_name)
-                    if old_name != source_old_name:
-                        name = old_name
+            with override("en"):
+                source_name = force_str(country_name)
+            country_name = force_str(country_name)
+            if country_name == source_name:
+                for fallback_name in fallback_names:
+                    with override("en"):
+                        source_fallback_name = force_str(fallback_name)
+                    fallback_name = force_str(fallback_name)
+                    if fallback_name != source_fallback_name:
+                        country_name = fallback_name
                         break
         else:
-            name = force_str(name)
-        return CountryTuple(code, name)
+            country_name = force_str(country_name)
+        return CountryTuple(code, country_name)
 
     def __iter__(self):
         """
@@ -201,22 +278,6 @@ class Countries(CountriesBase):
         # Yield countries that should be displayed first.
         countries_first = (self.translate_pair(code) for code in self.countries_first)
 
-        # Define the sorting method.
-        if pyuca:
-            collator = pyuca.Collator()
-
-            # Use UCA sorting if it's available.
-            def sort_key(item):
-                return collator.sort_key(item[1])
-        else:
-            # Cheap and dirty method to sort against ASCII characters only.
-            def sort_key(item):
-                return (
-                    unicodedata.normalize("NFKD", item[1])
-                    .encode("ascii", "ignore")
-                    .decode("ascii")
-                )
-
         if self.get_option("first_sort"):
             countries_first = sorted(countries_first, key=sort_key)
 
@@ -226,7 +287,7 @@ class Countries(CountriesBase):
         if self.countries_first:
             first_break = self.get_option("first_break")
             if first_break:
-                yield ("", force_str(first_break))
+                yield CountryTuple("", force_str(first_break))
 
         # Force translation before sorting.
         ignore_first = None if self.get_option("first_repeat") else self.countries_first
@@ -240,50 +301,55 @@ class Countries(CountriesBase):
         for item in sorted(countries, key=sort_key):
             yield item
 
-    def alpha2(self, code):
+    def alpha2(self, code: CountryCode) -> str:
         """
-        Return the two letter country code when passed any type of ISO 3166-1
+        Return the normalized country code when passed any type of ISO 3166-1
         country code.
 
+        Overridden countries objects may actually have country codes that are
+        not two characters (for example, "GB-WLS"), so the returned length of
+        the code is not guaranteed.
+
         If no match is found, returns an empty string.
         """
-        code = force_str(code).upper()
-        if code.isdigit():
-            lookup_code = int(code)
+        find: Optional[Callable]
+        code_str = force_str(code).upper()
+        if code_str.isdigit():
+            lookup_numeric = int(code_str)
 
             def find(alt_codes):
-                return alt_codes[1] == lookup_code
+                return alt_codes[1] == lookup_numeric
 
-        elif len(code) == 3:
-            lookup_code = code
+        elif len(code_str) == 3:
+            lookup_alpha3 = code_str
 
             def find(alt_codes):
-                return alt_codes[0] == lookup_code
+                return alt_codes[0] == lookup_alpha3
 
         else:
             find = None
         if find:
-            code = None
+            code_str = ""
             for alpha2, alt_codes in self.alt_codes.items():
                 if find(alt_codes):
-                    code = alpha2
+                    code_str = alpha2
                     break
-        if code in self.countries:
-            return code
+        if code_str in self.countries:
+            return code_str
         return ""
 
-    def name(self, code):
+    def name(self, code: CountryCode) -> str:
         """
         Return the name of a country, based on the code.
 
         If no match is found, returns an empty string.
         """
-        code = self.alpha2(code)
-        if code not in self.countries:
+        alpha2 = self.alpha2(code)
+        if alpha2 not in self.countries:
             return ""
-        return self.translate_pair(code)[1]
+        return self.translate_pair(alpha2)[1]
 
-    def by_name(self, country, language="en"):
+    def by_name(self, country: str, language: str = "en") -> str:
         """
         Fetch a country's ISO3166-1 two letter country code from its name.
 
@@ -297,29 +363,38 @@ class Countries(CountriesBase):
             countries may change over time.
         """
         with override(language):
-            for code, name in self:
-                if name.lower() == country.lower():
-                    return code
-                if code in self.OLD_NAMES:
-                    for old_name in self.OLD_NAMES[code]:
-                        if old_name.lower() == country.lower():
+            for code, check_country in self.countries.items():
+                if isinstance(check_country, dict):
+                    if "names" in check_country:
+                        check_names: List[str] = check_country["names"]
+                    else:
+                        check_names = [check_country["name"]]
+                else:
+                    check_names = [check_country]
+                for name in check_names:
+                    if name.lower() == country.lower():
+                        return code
+                if code in self.shadowed_names:
+                    for shadowed_name in self.shadowed_names[code]:
+                        if shadowed_name.lower() == country.lower():
                             return code
         return ""
 
-    def alpha3(self, code):
+    def alpha3(self, code: CountryCode):
         """
         Return the ISO 3166-1 three letter country code matching the provided
         country code.
 
         If no match is found, returns an empty string.
         """
-        code = self.alpha2(code)
+        alpha2 = self.alpha2(code)
         try:
-            return self.alt_codes[code][0]
+            alpha3 = self.alt_codes[alpha2][0]
         except KeyError:
-            return ""
+            alpha3 = ""
+        return alpha3 or ""
 
-    def numeric(self, code, padded=False):
+    def numeric(self, code: Union[str, int, None], padded=False):
         """
         Return the ISO 3166-1 numeric country code matching the provided
         country code.
@@ -329,15 +404,27 @@ class Countries(CountriesBase):
         :param padded: Pass ``True`` to return a 0-padded three character
             string, otherwise an integer will be returned.
         """
-        code = self.alpha2(code)
+        alpha2 = self.alpha2(code)
         try:
-            num = self.alt_codes[code][1]
+            num = self.alt_codes[alpha2][1]
         except KeyError:
+            num = None
+        if num is None:
             return None
         if padded:
             return "%03d" % num
         return num
 
+    def ioc_code(self, code: CountryCode) -> str:
+        """
+        Return the International Olympic Committee three letter code matching
+        the provided ISO 3166-1 country code.
+
+        If no match is found, returns an empty string.
+        """
+        alpha2 = self.alpha2(code)
+        return self.ioc_codes.get(alpha2, "")
+
     def __len__(self):
         """
         len() used by several third party applications to calculate the length
