@@ -1,6 +1,7 @@
-import pkg_resources
+from typing import Any, Iterable, Tuple, Type, Union
 from urllib import parse as urlparse
 
+import pkg_resources
 from django import forms
 from django.contrib.admin.filters import FieldListFilter
 from django.core import checks, exceptions
@@ -9,7 +10,7 @@ from django.utils.encoding import force_str
 from django.utils.functional import lazy
 from django.utils.html import escape as escape_html
 
-from django_countries import countries, filters, ioc_data, widgets
+from django_countries import Countries, countries, filters, ioc_data, widgets
 from django_countries.conf import settings
 
 EXTENSIONS = dict(
@@ -69,13 +70,13 @@ class Country:
         return hash(force_str(self))
 
     def __repr__(self):
-        args = ["code={country.code!r}"]
+        args = [f"code={self.code!r}"]
         if self.flag_url is not None:
-            args.append("flag_url={country.flag_url!r}")
+            args.append(f"flag_url={self.flag_url!r}")
         if self._str_attr != "code":
-            args.append("str_attr={country._str_attr!r}")
-        args = ", ".join(args).format(country=self)
-        return "{name}({args})".format(name=self.__class__.__name__, args=args)
+            args.append(f"str_attr={self._str_attr!r}")
+        args = ", ".join(args)
+        return f"{self.__class__.__name__}({args})"
 
     def __bool__(self):
         return bool(self.code)
@@ -138,7 +139,8 @@ class Country:
         """
         if not self.code:
             return ""
-        return "flag-sprite flag-{} flag-_{}".format(*self.code.lower())
+        x, y = self.code.lower()
+        return f"flag-sprite flag-{x} flag-_{y}"
 
     @property
     def unicode_flag(self):
@@ -171,7 +173,7 @@ class Country:
 
     @property
     def ioc_code(self):
-        return ioc_data.ISO_TO_IOC.get(self.code, "")
+        return self.countries.ioc_code(self.code)
 
     def __getattr__(self, attr):
         if attr in EXTENSIONS:
@@ -212,6 +214,7 @@ class CountryDescriptor:
         return Country(
             code=code,
             flag_url=self.field.countries_flag_url,
+            str_attr=self.field.countries_str_attr,
             custom_countries=self.field.countries,
         )
 
@@ -229,11 +232,17 @@ class LazyChoicesMixin(widgets.LazyChoicesMixin):
         self.widget.choices = value
 
 
+_Choice = Tuple[Any, str]
+_ChoiceNamedGroup = Tuple[str, Iterable[_Choice]]
+_FieldChoices = Iterable[Union[_Choice, _ChoiceNamedGroup]]
+
+
 class LazyTypedChoiceField(LazyChoicesMixin, forms.TypedChoiceField):
     """
     A form TypedChoiceField that respects choices being a lazy object.
     """
 
+    choices: Any
     widget = widgets.LazySelect
 
 
@@ -242,6 +251,7 @@ class LazyTypedMultipleChoiceField(LazyChoicesMixin, forms.TypedMultipleChoiceFi
     A form TypedMultipleChoiceField that respects choices being a lazy object.
     """
 
+    choices: Any
     widget = widgets.LazySelectMultiple
 
 
@@ -254,9 +264,10 @@ class CountryField(CharField):
     descriptor_class = CountryDescriptor
 
     def __init__(self, *args, **kwargs):
-        countries_class = kwargs.pop("countries", None)
+        countries_class: Type[Countries] = kwargs.pop("countries", None)
         self.countries = countries_class() if countries_class else countries
         self.countries_flag_url = kwargs.pop("countries_flag_url", None)
+        self.countries_str_attr = kwargs.pop("countries_str_attr", "code")
         self.blank_label = kwargs.pop("blank_label", None)
         self.multiple = kwargs.pop("multiple", None)
         kwargs["choices"] = self.countries
@@ -265,10 +276,16 @@ class CountryField(CharField):
             # changes in the multiple CountryField fields when new countries are
             # added to the available countries dictionary.
             if self.multiple:
-                kwargs["max_length"] = len(self.countries) * 3 - 1
+                kwargs["max_length"] = (
+                    len(self.countries)
+                    - 1
+                    + sum(len(code) for code in self.countries.countries)
+                )
             else:
-                kwargs["max_length"] = 2
-        super(CharField, self).__init__(*args, **kwargs)
+                kwargs["max_length"] = max(
+                    len(code) for code in self.countries.countries
+                )
+        super().__init__(*args, **kwargs)
 
     def check(self, **kwargs):
         errors = super().check(**kwargs)
@@ -340,8 +357,8 @@ class CountryField(CharField):
         Not including the ``blank_label`` property, as this isn't database
         related.
         """
-        name, path, args, kwargs = super().deconstruct()
-        kwargs.pop("choices")
+        name, path, args, kwargs = super(CharField, self).deconstruct()
+        kwargs.pop("choices", None)
         if self.multiple:  # multiple determines the length of the field
             kwargs["multiple"] = self.multiple
         if self.countries is not countries:
