@@ -1,6 +1,6 @@
 from django.contrib import admin
-from django.utils.encoding import force_text
-from django.utils.translation import ugettext_lazy as _
+from django.utils.encoding import force_str
+from django.utils.translation import gettext_lazy as _
 
 
 class CountryFilter(admin.FieldListFilter):
@@ -8,7 +8,8 @@ class CountryFilter(admin.FieldListFilter):
     A country filter for Django admin that only returns a list of countries
     related to the model.
     """
-    title = _('Country')
+
+    title = _("Country")
 
     def expected_parameters(self):
         return [self.field.name]
@@ -16,17 +17,17 @@ class CountryFilter(admin.FieldListFilter):
     def choices(self, changelist):
         value = self.used_parameters.get(self.field.name)
         yield {
-            'selected': value is None,
-            'query_string': changelist.get_query_string(
-                {}, [self.field.name]),
-            'display': _('All'),
+            "selected": value is None,
+            "query_string": changelist.get_query_string({}, [self.field.name]),
+            "display": _("All"),
         }
         for lookup, title in self.lookup_choices(changelist):
             yield {
-                'selected': value == force_text(lookup),
-                'query_string': changelist.get_query_string(
-                    {self.field.name: lookup}, []),
-                'display': title,
+                "selected": value == force_str(lookup),
+                "query_string": changelist.get_query_string(
+                    {self.field.name: lookup}, []
+                ),
+                "display": title,
             }
 
     def lookup_choices(self, changelist):
@@ -34,7 +35,8 @@ class CountryFilter(admin.FieldListFilter):
         codes = set(
             qs.distinct()
             .order_by(self.field.name)
-            .values_list(self.field.name, flat=True))
+            .values_list(self.field.name, flat=True)
+        )
         for k, v in self.field.get_choices(include_blank=False):
             if k in codes:
                 yield k, v
