@@ -133,13 +133,60 @@ class TestCountriesObject(BaseTest):
         with self.settings(COUNTRIES_OVERRIDE={"XX": "Neverland"}):
             self.assertEqual(countries.alpha2("XX"), "XX")
 
+    def test_ioc_code(self):
+        self.assertEqual(countries.ioc_code("BS"), "BAH")
+
+    def test_ioc_code_override(self):
+        with self.settings(
+            COUNTRIES_OVERRIDE={
+                "BS": "Bahamas in Pajamas",
+                "AU": None,
+                "NZ": {"ioc_code": ""},
+                "XX": "Neverland",
+                "US": {"ioc_code": "XXX"},
+                "XK": {"name": "Kosovo", "ioc_code": "KOS"},
+            }
+        ):
+            self.assertEqual(
+                countries.ioc_code("BS"),
+                "BAH",
+                "Should still use built-in code if only name changed",
+            )
+            self.assertEqual(
+                countries.ioc_code("AU"),
+                "",
+                "Should be empty since country was marked not present",
+            )
+            self.assertEqual(
+                countries.ioc_code("NZ"),
+                "",
+                "Should be empty since country exists but IOC code cleared",
+            )
+            self.assertEqual(
+                countries.ioc_code("XX"),
+                "",
+                "Should be empty for a custom country with no IOC code",
+            )
+            self.assertEqual(
+                countries.ioc_code("US"), "XXX", "Should use provided custom IOC code"
+            )
+            self.assertEqual(
+                countries.ioc_code("XK"),
+                "KOS",
+                "Should use IOC code for a custom country that provides a code",
+            )
+
     def test_fetch_by_name(self):
-        code = countries.by_name("United States of America")
-        self.assertEqual(code, "US")
+        code = countries.by_name("Brunei")
+        self.assertEqual(code, "BN")
+
+    def test_fetch_by_name_official(self):
+        code = countries.by_name("brunei darussalam")
+        self.assertEqual(code, "BN")
 
     def test_fetch_by_name_case_insensitive(self):
-        code = countries.by_name("United states of America")
-        self.assertEqual(code, "US")
+        code = countries.by_name("bRuNeI")
+        self.assertEqual(code, "BN")
 
     def test_fetch_by_name_old(self):
         code = countries.by_name("Czech Republic")
@@ -156,6 +203,17 @@ class TestCountriesObject(BaseTest):
     def test_fetch_by_name_no_match(self):
         self.assertEqual(countries.by_name("Neverland"), "")
 
+    def test_fetch_by_name_custom(self):
+        with self.settings(
+            COUNTRIES_ONLY={
+                "AU": {"name": "Oz"},
+                "NZ": {"names": ["New Zealand", "Hobbiton"]},
+            }
+        ):
+            self.assertEqual(countries.by_name("Oz"), "AU")
+            self.assertEqual(countries.by_name("New Zealand"), "NZ")
+            self.assertEqual(countries.by_name("Hobbiton"), "NZ")
+
     def test_multiple_labels(self):
         with self.settings(
             COUNTRIES_ONLY={
@@ -236,30 +294,85 @@ class CountriesFirstTest(BaseTest):
 
     def test_sorted_countries_first_english(self):
         with self.settings(
-            COUNTRIES_FIRST=["GB", "AF", "DK"], COUNTRIES_FIRST_SORT=True
+            COUNTRIES_FIRST=["NZ", "CA", "YE"], COUNTRIES_FIRST_SORT=True
         ):
             countries_list = list(countries)
             sorted_codes = [item[0] for item in countries_list[:3]]
-            self.assertEqual(["AF", "DK", "GB"], sorted_codes)
+            # Canada, New Zealand, Yemen
+            self.assertEqual(["CA", "NZ", "YE"], sorted_codes)
 
     def test_unsorted_countries_first_english(self):
         with self.settings(
-            COUNTRIES_FIRST=["GB", "AF", "DK"], COUNTRIES_FIRST_SORT=False
+            COUNTRIES_FIRST=["NZ", "CA", "YE"], COUNTRIES_FIRST_SORT=False
         ):
             countries_list = list(countries)
             unsorted_codes = [item[0] for item in countries_list[:3]]
-            self.assertEqual(["GB", "AF", "DK"], unsorted_codes)
+            self.assertEqual(["NZ", "CA", "YE"], unsorted_codes)
 
-    def test_sorted_countries_first_arabic(self):
+    def test_sorted_countries_first_translated(self):
         with self.settings(
-            COUNTRIES_FIRST=["GB", "AF", "DK"], COUNTRIES_FIRST_SORT=True
+            COUNTRIES_FIRST=["NZ", "CA", "YE"], COUNTRIES_FIRST_SORT=True
         ):
             lang = translation.get_language()
             translation.activate("eo")
             try:
                 countries_list = list(countries)
                 sorted_codes = [item[0] for item in countries_list[:3]]
-                self.assertEqual(["AF", "GB", "DK"], sorted_codes)
+                # Jemeno, Kanado, Nov-Zelando
+                self.assertEqual(["YE", "CA", "NZ"], sorted_codes)
+            finally:
+                translation.activate(lang)
+
+    def test_translation_fallback_from_common_name(self):
+        trans_fall_countries = custom_countries.TranslationFallbackCountries()
+        self.assertEqual(trans_fall_countries.name("YE"), "YYYemen")
+        lang = translation.get_language()
+        translation.activate("eo")
+        try:
+            self.assertEqual(trans_fall_countries.name("YE"), "Jemeno")
+        finally:
+            translation.activate(lang)
+
+    def test_translation_fallback_from_old_name(self):
+        trans_fall_countries = custom_countries.TranslationFallbackCountries()
+        trans_fall_countries.countries["NZ"] = "Old Zealand"
+
+        self.assertEqual(trans_fall_countries.name("NZ"), "Old Zealand")
+        lang = translation.get_language()
+        translation.activate("eo")
+        try:
+            self.assertEqual(trans_fall_countries.name("NZ"), "Nov-Zelando")
+        finally:
+            translation.activate(lang)
+
+    def test_translation_fallback_no_override(self):
+        lang = translation.get_language()
+        translation.activate("eo")
+
+        try:
+            self.assertEqual(countries.name("NZ"), "Nov-Zelando")
+            self.assertEqual(countries.name("YE"), "Jemeno")
+
+            with self.settings(COUNTRIES_OVERRIDE={"NZ": "Hobbiton", "YE": "YYemen"}):
+                del countries.countries
+                self.assertEqual(countries.name("NZ"), "Hobbiton")
+                self.assertEqual(countries.name("YE"), "YYemen")
+        finally:
+            translation.activate(lang)
+
+    def test_translation_fallback_override_names(self):
+        with self.settings(
+            COUNTRIES_OVERRIDE={
+                "NZ": {"names": ["Hobbiton", translation.gettext_lazy("New Zealand")]}
+            }
+        ):
+
+            self.assertEqual(countries.name("NZ"), "Hobbiton")
+            lang = translation.get_language()
+            translation.activate("eo")
+
+            try:
+                self.assertEqual(countries.name("NZ"), "Nov-Zelando")
             finally:
                 translation.activate(lang)
 
